#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# HTML writer for IRCMemoBot
#
# Copyright(C) 2005-, Atzm WATANABE <sitosito@p.chan.ne.jp>
#
# $Id: $

import fcntl, os, re, sha, urllib
from Cheetah.Template import Template
from RSS              import TrackingChannel
from ordereddict      import OrderedDictionary
from common           import PKG, VER, ns, unicoding, ununicoding, htmlspecialchars, get_all_nick

class HTMLizer:
	RELATED_BASE = 'http://beta.search.msn.co.jp/results.aspx?format=rss&q=link:%s'

	def __init__(self, config):
		self._config = config

	def write(self, rss, rss_place, page_title, author_name, html_path):
		template              = Template(file(self._config.HTML_TEMPLATE).read())
		template.author       = htmlspecialchars(self._config.AUTHOR)
		template.sanchor      = htmlspecialchars(self._config.SANCHOR)
		template.canchor      = htmlspecialchars(self._config.CANCHOR)
		template.tdiary_base  = self._config.TDIARY_THEME_BASE_URL
		template.tdiary_theme = self._config.TDIARY_THEME_URL
		template.pkg          = PKG
		template.ver          = VER

		template.page_title   = htmlspecialchars(page_title)
		template.author_name  = htmlspecialchars(author_name)
		template.rss_place    = rss_place

		if self._config.INDIVIDUAL_MEMO:
			template.nicknames = self.get_nicknames()
		else:
			template.nicknames = []

		template.entries = self.gen_entries(rss)

		fileobj = file(html_path, 'w+')
		fcntl.flock(fileobj.fileno(), fcntl.LOCK_EX)
		fileobj.write(template.__str__())
		fcntl.flock(fileobj.fileno(), fcntl.LOCK_UN)
		fileobj.close()

	REGEX_SPLITTER = re.compile('T|\+')
	def gen_entries(self, rss):
		entries  = OrderedDictionary()
		for key in rss.data[(ns.rss10, 'items')]:
			for item in rss.data[key[0]]:
				title, desc, link, date  = self.get_elements_from_item(item)
				day, localtime, timezone = self.REGEX_SPLITTER.split(date)
				related = self.get_related_links(link)
				if not entries.has_key(day): entries[day] = []

				entries[day].append([htmlspecialchars(ununicoding(title)),
									 htmlspecialchars(ununicoding(desc)),
									 htmlspecialchars(ununicoding(link)),
									 htmlspecialchars(ununicoding(localtime)),
									 related])
		return entries

	def get_elements_from_item(self, item):
		elements = [
			item.get((ns.rss10, 'title'), ''),
			item.get((ns.rss10, 'description'), ''),
			item.get((ns.rss10, 'link'), ''),
			item.get((ns.dc, 'date'), ''),
			]
		for i in range(len(elements)):
			elements[i] = unicoding(elements[i])
		return elements

	def get_related_links(self, url):
		ret = []
		filename = os.path.join(self._config.RSS_CACHE_DIR,
								'cache.%s.rdf' % sha.new(url).hexdigest())

		rss = TrackingChannel()
		if not os.path.exists(filename):
			filename, headers = urllib.urlretrieve(self.RELATED_BASE % url, filename)
		rss.parse(filename)

		for i in rss.listItems():
			if not i[0]:
				continue

			title = rss.get(i[0])[0][(ns.rss10, 'title')]
			link  = rss.get(i[0])[0][(ns.rss10, 'link')]
			desc  = rss.get(i[0])[0][(ns.rss10, 'description')]
			if not title:
				title = link
			if not desc:
				desc = ''

			ret.append([htmlspecialchars(title),
						htmlspecialchars(desc),
						htmlspecialchars(link)])

		return ret

	def get_nicknames(self):
		return [(nick, '%s.html' % nick) for nick in get_all_nick(self._config.OUTPUT_RSS_PATH)]
