# -*- coding: utf-8 -*-
#
# Process Scheduler for each commands
#
# Copyright(C) 2005-, Atzm WATANABE <sitosito@p.chan.ne.jp>
#
# $Id: $

from ordereddict import OrderedDictionary

class Scheduler:
	def __init__(self):
		"""
		term and proc are called in:
			1. term:
				1.1. _before_TERMNAME_term
				1.2. proc:
					1.2.1. _before_PROCNAME
					1.2.2. _PROCNAME
					1.2.3. _after_PROCNAME
				1.3. _after_TERMNAME_term
		"""
		self._term = OrderedDictionary()

# -- Utilities for scheduling
	def append_term(self, name, procs):
		"""append proc to 'name' term"""
		name = str(name)
		if __debug__ and self._term.has_key(name):
			sys.stderr.write("!!! Warning: term '%s' was already defined. overwrite this?\n" % name)
			sys.stderr.write("!!!          if you want to modify this term, should use 'insert_proc' !\n")
		self._term[name] = list(procs)

	def remove_term(self, name):
		"""remove 'name' term"""
		try:
			del self._term[str(name)]
		except KeyError:
			pass

	def append_proc(self, term_name, proc_name):
		"""append 'proc_name' proc to 'term_name' term"""
		self.insert_proc(term_name, proc_name, len(self._term))

	def insert_proc(self, term_name, proc_name, index=0):
		"""insert 'proc_name' proc to 'term_name' term, to 'position'"""
		try:
			self._term[str(term_name)].insert(index, str(proc_name))
		except KeyError:
			pass

	def remove_proc(self, term_name, proc_name):
		"""remove 'proc_name' proc from 'term_name' term"""
		try:
			tmp = self._term[str(term_name)]
			while tmp.count(str(proc_name)):
				del tmp[tmp.index(str(proc_name))]
		except KeyError:
			pass

# -- Structure
	def _default(self, c, e, msg, nick, channel): return True
	def __call__(self, c, e, msg, nick, channel):
		"""
		execute scheduled processes
		if proc returns False, just breaks process loop
		"""
		for term, procs in self._term.items():
			method = getattr(self, '_before_%s_term' % term, self._default)
			if not (callable(method) and method(c, e, msg, nick, channel)):
				c.notice(nick, 'An error has occured processing %s' % method)
				return False

			for proc in procs:
				for name in ['_before', '', '_after']:
					method = getattr(self, '%s_%s' % (name, proc), self._default)
					if not (callable(method) and method(c, e, msg, nick, channel)):
						c.notice(nick, 'An error has occured processing %s' % method)
						return False

			method = getattr(self, '_after_%s_term' % term, self._default)
			if not (callable(method) and method(c, e, msg, nick, channel)):
				c.notice(nick, 'An error has occured processing %s' % method)
				return False

		return True
