import urllib, sys, re

from HTMLParser import HTMLParser, HTMLParseError
from common     import unicoding, ununicoding

class StopParsing(Exception):
	pass

class SiteParser(HTMLParser):
	def __init__(self, url):
		self.url = url
		self.stripper = re.compile('\s+')
		self.title_flag = False
		self.site_title = []
		HTMLParser.__init__(self)

	def reset(self):
		HTMLParser.reset(self)
		self.title_flag = False
		self.site_title = []

	def handle_data(self, data):
		if self.title_flag:
			self.site_title.append(data)

	def handle_charref(self, ref):
		if self.title_flag:
			self.site_title.append('&#%s;' % ref)

	def handle_entityref(self, name):
		if self.title_flag:
			self.site_title.append('&%s;' % name)

	def handle_starttag(self, tag, attrs):
		if self.title_flag:
			raise StopParsing
		if tag == 'title':
			self.title_flag = True

	def handle_endtag(self, tag):
		if tag == 'title':
			raise StopParsing

	def get_title(self):
		fobj = urllib.urlopen(self.url)
		try:
			for line in fobj:
				line = unicoding(line.strip())
				try:
					self.feed(line)
				except HTMLParseError:
					self.reset()
		except StopParsing:
			pass
		return self.stripper.sub(' ', ''.join(self.site_title)).strip()

def main(url):
	sp = SiteParser(url)
	title = sp.get_title()
	print ununicoding(title, 'euc-jp')

if __name__ == '__main__':
	url = 'http://www.cnn.com/'
	if len( sys.argv ) > 1:
		url = sys.argv[1]
	main(url)
