#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Super Class for IRCMemoBot
#
# Copyright(C) 2005-, Atzm WATANABE <sitosito@p.chan.ne.jp>
#
# $Id: $

import string, re, glob, os, sys, time, traceback
try:    from cStringIO import StringIO
except: from StringIO  import StringIO

from ircbot import SingleServerIRCBot
from irclib import nm_to_n
from common import IRC_ENCODING, unicoding, ununicoding

PLUGINS = {}
for module_name in glob.glob('plugins/plugin_*.py'):
	upper_module_name, lower_module_name = os.path.split(module_name)
	lower_module_name = lower_module_name[:-3]
	full_module_name  = '%s.%s' % (upper_module_name, lower_module_name)

	upper_module = __import__(full_module_name)
	lower_module = getattr(upper_module, lower_module_name)
	constructor  = getattr(lower_module, lower_module_name)

	PLUGINS[lower_module_name] = constructor

class IRCEventHandler(SingleServerIRCBot):
	def __init__(self, channel, nickname, server, port=6667):
		if type(channel) is list:
			self.channel = channel
		else:
			self.channel = [channel]

		SingleServerIRCBot.__init__(self, [(server, port)], nickname, nickname)

		for name, constructor in PLUGINS.items():
			setattr(self, name, constructor(self.connection, self.channel))
			if __debug__:
				c, n = name.split('_')
				print '* Loaded %s as %s' % (n, c)

	def on_welcome(self, c, e):
		for ch in self.channel:
			c.join(ch)
			if __debug__:
				print '* Joined %s' % ch

	def on_nicknameinuse(self, c, e):
		c.nick(c.get_nickname() + '_')

	def on_privmsg(self, c, e):
		return self.on_pubmsg(c, e)

	def on_pubmsg(self, c, e):
		msg = e.arguments()[0]
		if msg[0] != '!': # commands' first char must be '!'
			return False

		nick    = nm_to_n(e.source())
		channel = e.target()
		msg     = unicoding(msg)

		return self.process_msg(c, e, msg, nick, channel)

	def process_msg(self, c, e, msg, nick, channel):
		pass

	def start(self):
		sys.excepthook = self._excepthook
		try:
			SingleServerIRCBot.start(self)
		except KeyboardInterrupt:
			self.stop()

	def stop(self):
		for name in PLUGINS.keys():
			try:
				getattr(self, name).stop()
			except AttributeError:
				pass

	def _excepthook(self, t, v, tb):
		self.stop()

		channels = ','.join(self.channel)
		buf = StringIO()
		traceback.print_exception(t, v, tb, None, buf)
		buf = buf.getvalue()
		sys.stderr.write(buf + '\n')

		lines = re.sub('".*/(.*?)"', r'"\1"', buf).split('\n')
		if len(lines) > 6:
			lines = lines[0:3] + ['...snip...'] + lines[-4:-1]

		for line in lines + ['(;_;)/~']:
			self.connection.privmsg(channels, line)
			time.sleep(1)
		time.sleep(1) # Network delay?
		sys.exit(1)
