#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# IRCMemoBot
#   Memo syntax:
#     !memo   <URL> <title> <comment>
#     !tmemo  <URL> <comment>
#     !modify <URL> <title> <comment>
#     !load   <rss|config>
#     !write  <rss|html>
#     !del
#
# Copyright(C) 2005-, Atzm WATANABE <sitosito@p.chan.ne.jp>
#
# $Id: $

import os, glob

from irceventhandler       import IRCEventHandler
from ircmemoconf           import IRCMemoConf
from logger                import Logger
from common                import ununicoding
from commands.supercommand import SuperCommand

COMMANDS = {}
for module_name in glob.glob('commands/command_*.py'):
	upper_module_name, lower_module_name = os.path.split(module_name)
	lower_module_name = lower_module_name[:-3]
	full_module_name  = '%s.%s' % (upper_module_name, lower_module_name)

	upper_module = __import__(full_module_name)
	lower_module = getattr(upper_module, lower_module_name)
	constructor  = getattr(lower_module, lower_module_name)

	assert issubclass(constructor, SuperCommand), '%s is not subclass of SuperCommand.' % constructor
	COMMANDS[lower_module_name] = constructor

class IRCMemoBot(IRCEventHandler):
	def __init__(self):
		config = IRCMemoConf()
		self.logger = Logger(config)

		for name, constructor in COMMANDS.items():
			setattr(self, name, constructor(config))
			if __debug__:
				c, n = name.split('_')
				print '* Loaded %s as %s' % (n, c)

		IRCEventHandler.__init__(self, config.CHANNELS, config.BOT_NAME,
								 config.SERVER_NAME, config.SERVER_PORT)

	def process_msg(self, c, e, msg, nick, channel): # Dispatcher
		try:
			command = msg.split()[0][1:]
			method  = getattr(self, 'command_%s' % command)
			if not callable(method): raise AttributeError
		except:
			c.notice(nick, 'Command not found: %s' % ununicoding(msg, IRC_ENCODING))
			return False

		return method(c, e, msg, nick, channel) and self.logger.write(msg, nick, channel)


def main():
	bot = IRCMemoBot()
	bot.start()

if __name__ == '__main__':
	main()
