#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Config Loader for IRCMemoBot
#
# Copyright(C) 2005-, Atzm WATANABE <sitosito@p.chan.ne.jp>
#
# $Id: $

import os, sys, re
from common import ununicoding, expand_path

class IRCMemoConf:
	CONFIGFILE = expand_path('./ircmemobot.conf')

	def __init__(self):
		self.__trans_table = {
			self.__get_prefix_table: ['APPEND_TITLE_TABLE'],
			ununicoding: ['PAGE_TITLE', 'AUTHOR', 'AUTHOR_NAME', 'SANCHOR', 'CANCHOR'],
			expand_path: ['OUTPUT_HTML_PATH', 'OUTPUT_RSS_PATH',
						  'HTML_TEMPLATE', 'RSS_TEMPLATE', 'RSS_CACHE_DIR', 'LOG_DIR'],
			bool:        ['INDIVIDUAL_MEMO'],
			str:         ['SERVER_NAME', 'RSS_PLACE', 'STRIP_NICK',
						  'TDIARY_THEME_URL', 'TDIARY_THEME_BASE_URL'],
			int:         ['SERVER_PORT', 'SAVE_LIMIT'],
			list:        ['CHANNELS'],
			}
		self.load()

	def load(self):
		"""load config file"""
		execfile(self.CONFIGFILE, self.__dict__)

		for func, items in self.__trans_table.items():
			for item in items:
				setattr(self, item, func(getattr(self, item)))

 		for item in [self.RSS_CACHE_DIR, self.LOG_DIR]:
 			if not self.__check_dir(item): sys.exit(1)

	def __get_prefix_table(self, table_list):
		return [[re.compile(regex), title] for regex, title in table_list]

	def __check_dir(self, directory):
		if not os.path.isdir(directory):
			try:
				os.mkdir(directory)
			except OSError:
				sys.stderr.write('file already exists: %s\n' % directory)
				return False
		return True

if __name__ == '__main__':
	c = IRCMemoConf()
	for val in c._IRCMemoConf__trans_table.values():
		for v in val:
			print '%s: %s' % (v, getattr(c, v))
